#include-once

#include <AutoItConstants.au3>
#include <Date.au3>
#include <FileConstants.au3>
#include <File.au3>
#include <WinAPICom.au3>
#include <Array.au3>
#include <StringConstants.au3>

; #INDEX# =======================================================================================================================
; Title .........: The Backbone Automation UDF Library for AutoIt3
; Lib Version ...: v1.1.0.0
; AutoIt Version : 3.3.14.2
; Language ......: English
; Description ...: A collection of commonly used functions for creating AutoIt scripts for The Backbone EUM.
; Author(s) .....: The Backbone - The Netherlands
; Dll ...........:
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
; _EUM_DataClean
; _EUM_DataWrite
; _EUM_Init
; _EUM_InstanceStart
; _EUM_InstanceStop
; _EUM_LogInit
; _EUM_LogWrite
; _EUM_StepStart
; _EUM_StepStop
; _EUM_WatcherSet
; ===============================================================================================================================

; #VARIABLES# ===================================================================================================================
#Region Global Variables
Global $__g_aDataArray[0][7] ; two dimensional array with 7 columns, rows get added with the _ArrayAdd function from Array.au3

; Instance variables
Global $__g_sInstanceID = Null
Global $__g_oInstanceStartDate = Null
Global $__g_oInstanceTimer = Null
Global $__g_bInstanceState = True
Global $__g_iInstanceTime = Null

; Step variables
Global $__g_sStepID = Null
Global $__g_oStepStartDate = Null
Global $__g_oStepTimer = Null
Global $__g_bStepState = True
Global $__g_iStepTime = Null

; Other variables
Global $__g_sCollectorPath = ""
Global $__g_sWatcherNode
Global $__g_TimeInstance

; Log variables
Global Enum $_g_eLogDisabled, $_g_eLog2Console, $_g_eLog2File, $_g_eLog2Both
Global $__g_bLog2Console = False
Global $__g_bLog2File = False
Global $__g_sLogFilePath = ""
Global $__g_sLogFileName = ""

#EndRegion Global Variables
; ===============================================================================================================================

#Region Core functions
; #FUNCTION# ====================================================================================================================
; Name...........: _EUM_DataClean
; Description ...: Clean Global variables when instancestart is called without the previous instance is stopped.
; Syntax.........: _EUM_DataClean()
; Parameters ....: none
; Return values .: Success      -	1
;                  Failure      -	0
; Author ........: The Backbone - The Netherlands
; Modified.......: 31/05/2016
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _EUM_DataClean()

	$__g_sInstanceID = Null
	$__g_oInstanceStartDate = Null
	$__g_oInstanceTimer = Null
	$__g_bInstanceState = True
	$__g_iInstanceTime = Null

	$__g_sStepID = Null
	$__g_oStepStartDate = Null
	$__g_oStepTimer = Null
	$__g_bStepState = True
	$__g_iStepTime = Null

	If @error Then
		Return SetError(@error, 0, 0)
	Else
		Return 1
	EndIf

EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _EUM_DataWrite
; Description ...: Write the data to the eum file so the Module can import the data into the SCOM database
; Syntax.........: _EUM_DataWrite()
; Parameters ....: $bIgnoreInstance  -	An optional Boolean, True means instance data is ignored and removed
;										from the array and not written to file, default is false
; Return values .: Success      -	1
;                  Failure      -	0
; Author ........: The Backbone - The Netherlands
; Modified.......: 24/06/2016
; Remarks .......:
; Related .......: Array.au3
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _EUM_DataWrite($bIgnoreInstance = False)

	Local $sFileName, $hFileHandle, $sLine

	If Not (FileExists($__g_sCollectorPath)) Then
		Return 0
	EndIf

	; Set Guid as Filename
	$sFileName = _WinAPI_CreateGUID() &".eum"

	; Open file handle
	$hFileHandle = FileOpen($__g_sCollectorPath&$sFileName, 2)

	if($hFileHandle <> -1) Then
		$sLine = "Start data"

		; write data to file
		FileWriteLine($hFileHandle, $sLine)

		If $__g_oInstanceTimer <> Null Then
			_EUM_InstanceStop(False, "Stopped by UDF")
		EndIf

		If($bIgnoreInstance = True) Then
			For $i = 0 to UBound($__g_aDataArray) - 1
				;Check if column 3 is emtpy and delete instance from array
				If ($__g_aDataArray[$i][3] = "") Then
					_ArrayDelete($__g_aDataArray, $i)
				EndIf
			Next
		EndIf

		; Write Array to File with the right delimiter
		_FileWriteFromArray($hFileHandle,$__g_aDataArray,Default,Default,"|")

		$sLine = "End data"
		; write data to file
		FileWriteLine($hFileHandle, $sLine)
	EndIf

	FileClose ($hFileHandle)

	If @error Or $hFileHandle = -1 Then
		Return SetError(@error, 0, 0)
	Else
		Return 1
	EndIf

EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _EUM_Init
; Description ...: Initialize the CollectorPath, InstanceID and WatcherNode
; Syntax.........: _EUM_Init($_sCollectorPath, $_sInstanceID)
; Parameters ....: $_sCollectorPath	-	Path to the directory of the collector
; Return values .: Success      -	1
;                  Failure      -	0
; Author ........: The Backbone - The Netherlands
; Modified.......: 24/06/2016
; Remarks .......: This function also sets the FQDN of the WatcherNode.
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _EUM_Init($_sCollectorPath)

	; Fill global variables
	If $_sCollectorPath = "" Then

		Return 0

	Else

		if StringRight($_sCollectorPath, 1) <> "\" Then
			$__g_sCollectorPath = $_sCollectorPath & "\"
		Else
			$__g_sCollectorPath = $_sCollectorPath
		EndIf

		If @LogonDNSDomain = "" Then
			$__g_sWatcherNode = @ComputerName
		Else
			$__g_sWatcherNode = @ComputerName & "." & @LogonDNSDomain
		EndIf
		Return 1
	EndIf

EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _EUM_InstanceStart
; Description ...: Sets start date/time for the instance and start the Instance Timer
; Syntax.........: _EUM_InstanceStart()
; Parameters ....: $_sInstanceID	-	ID (string) of the instance
; Return values .: Success      -	1
;                  Failure      -	0
; Author ........: The Backbone - The Netherlands
; Modified.......: 31/05/2016
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _EUM_InstanceStart($_sInstanceID)

	; Clean any old information
	_EUM_DataClean()

	; Store the id en datetime the instance is started
	$__g_sInstanceID = $_sInstanceID
	$__g_oInstanceStartDate = _Now()

	; start timer instance
	$__g_oInstanceTimer = TimerInit()

	; Set the measured time to 0
	$__g_iInstanceTime = 0

	If @error Then
		Return SetError(@error, 0, 0)
	Else
		Return 1
	EndIf

EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _EUM_InstanceStop
; Description ...: Calculate time difference since instance start. Format to correct output and add data to array
; Syntax.........: _EUM_InstanceStop($_bState, $_sContext = "")
; Parameters ....: $_bState		-	The state of the instance, can be true of false
;				   $_sContext	-	An optional string to add additional information to the step, default is empty
; Return values .: Success      -	1
;                  Failure      -	0
; Author ........: The Backbone - The Netherlands
; Modified.......: 12/07/2016
; Remarks .......:
; Related .......: Array.au3
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _EUM_InstanceStop($_bState, $_sContext = "")

	Local $sFill

	; Remove unwanted characters and things like Linefeeds, Carriage Return
	$_sContext = _EUM_StripContext($_sContext)

	; Stop the Step if it is still active
	If $__g_oStepTimer <> Null Then
		_EUM_StepStop($__g_sStepID, False, "Stopped by UDF")
	EndIf

	; Calculate the time between start and stop only if the time isn't
	; based on the time of the steps
	If $__g_iInstanceTime = 0 Then
		$__g_iInstanceTime = Round(TimerDiff($__g_oInstanceTimer),0)
	EndIf

	; Set the state if it isn't correct already
	If $_bState = False Or $__g_bInstanceState = False Then
		; set $_bState to False in case only a step error occured
		$__g_bInstanceState = False
		$__g_iInstanceTime = -1
	EndIf

	; Set the correct output for writing to file
	$sFill = $__g_oInstanceStartDate & "|" & $__g_sWatcherNode & "|" & $__g_sInstanceID & "||" & $__g_iInstanceTime & "|" & $__g_bInstanceState & "|" & $_sContext
	_ArrayAdd($__g_aDataArray, $sFill)

	$__g_oInstanceTimer = Null

	If @error Then
		Return SetError(@error, 0, 0)
	Else
		Return 1
	EndIf

EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _EUM_LogInit
; Description ...: Initiates the log by setting the Path and Filename variable of the logfile and turns the log on or off,
;                  according to parameter given
; Syntax.........: _EUM_LogInit ($_iLogEnabled)
; Parameters ....: $_iLogEnabled -	Indication if the messages should be logged to console, file or both
;                                   Use the Enumeration variables:
;                                   $_g_eLogDisabled, $_g_eLog2Console, $_g_eLog2File, $_g_eLog2Both
; Return values .: Success      - 	1
;                  Failure      -	0
;                  @error       -   1 = Value is not an integer
;                                   2 = Integer is not one of the Enumeration values
; Author ........: The Backbone - The Netherlands
; Modified.......: 6/10/2016
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _EUM_LogInit ($_iLogEnabled)

	If IsInt ($_iLogEnabled) Then
		Switch $_iLogEnabled
			Case $_g_eLogDisabled
				$__g_bLog2Console = False
				$__g_bLog2File    = False
			Case $_g_eLog2Console
				$__g_bLog2Console = True
				$__g_bLog2File    = False
			Case $_g_eLog2File
				$__g_bLog2Console = False
				$__g_bLog2File    = True
			Case $_g_eLog2Both
				$__g_bLog2Console = True
				$__g_bLog2File    = True
			Case Else
				; Value is not one of the Enumeration values. Set @error to "2"
				Return SetError(2, 0, 0)
		EndSwitch
	Else
		; Value is not an integer. Set @error to "1"
		Return SetError(1, 0, 0)
	EndIf

	; Set Filename and Filepath for logfile if Log2File is enabled
	If $__g_bLog2File Then
		$__g_sLogFilePath = @ScriptDir & "\"
		$__g_sLogFileName = StringLeft (@ScriptName, StringInStr (@ScriptName, ".", 0, -1) - 1) & "_" & @ComputerName & ".log"
	EndIf

	Return 1
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _EUM_LogWrite
; Description ...: Send a log message to console, file or both.
; Syntax.........: _EUM_LogWrite ($_sMessage)
; Parameters ....: $_sMessage   -	Message that needs to be logged.
; Return values .: Success      - 	1
;                  Failure      -	0
;                  @error       -   1 = Logging is turned off
;                                   2 = Unable to write to logfile
; Author ........: The Backbone - The Netherlands
; Modified.......: 6/10/2016
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _EUM_LogWrite ($_sMessage)

	Local $sDateTime = @YEAR & "/" & @MON & "/" & @MDAY& " " & @HOUR & ":" & @MIN & ":" & @SEC

	If (Not $__g_bLog2Console) And (Not $__g_bLog2File) Then
		Return SetError(1, 0, 0)
	EndIf

	If $__g_bLog2Console Then
		ConsoleWrite ($sDateTime & " - " & $_sMessage & @CRLF)
	EndIf

	If $__g_bLog2File Then
		If FileWriteLine ($__g_sLogFilePath & $__g_sLogFileName, $sDateTime & " - " & $_sMessage & @CRLF) = 0 Then
			Return SetError(2, 0, 0)
		EndIf
	EndIf

	Return 1
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _EUM_StepStart
; Description ...: Sets start date/time for the step and start the step timer
; Syntax.........: _EUM_StepStart($_sStepID)
; Parameters ....: $_sStepID	-	Id of the step
; Return values .: Success      -	1
;                  Failure      -	0
; Author ........: The Backbone - The Netherlands
; Modified.......: 26/05/2016
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _EUM_StepStart($_sStepID)

	if ($__g_oStepTimer <> Null) Then
		_EUM_StepStop($__g_sStepID, False, "Stopped by UDF")
	EndIf

	; Set start date/time for the step
	$__g_oStepStartDate = _Now()
	$__g_bStepState = True

	; start timer step
	$__g_oStepTimer = TimerInit()

	; fill global variable with StepID
	$__g_sStepID = $_sStepID

	If @error Then
		Return SetError(@error, 0, 0)
	Else
		Return 1
	EndIf

EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _EUM_StepStop
; Description ...: Calculate time difference since step start. Format to correct output and add data to array
; Syntax.........: _EUM_StepStop($_sStepID, $_bState, $_sContext = "")
; Parameters ....: $_sStepID 	-	ID(string) of the Step
;				   $_bState		-	The state of the step, can be True or False
;				   $_sContext	-	An optional string to add additional information to the step, default is empty
; Return values .: Success      -	1
;                  Failure      -	0
; Author ........: The Backbone - The Netherlands
; Modified.......: 12/07/2016
; Remarks .......:
; Related .......: Array.au3
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _EUM_StepStop($_sStepID, $_bState, $_sContext = "")

	Local $sFill

	; Remove unwanted characters and things like Linefeeds, Carriage Return
	$_sContext = _EUM_StripContext($_sContext)

	; Calculate the time between start and stop
	$__g_iStepTime = Round(TimerDiff($__g_oStepTimer),0)

	;Add the time to the instance time
	$__g_iInstanceTime += $__g_iStepTime

	; Set the state
	$__g_bStepState = $_bState

	If $_bState = False Then
		$__g_iStepTime = -1
		$__g_bInstanceState = False
	EndIf

	$__g_oStepTimer = Null

	; Set the correct output for writing to file
	$sFill = $__g_oStepStartDate & "|" & $__g_sWatcherNode & "|" & $__g_sInstanceID & "|" & $_sStepID & "|" & $__g_iStepTime & "|" & $__g_bStepState & "|" & $_sContext
	_ArrayAdd($__g_aDataArray, $sFill)

	If @error Then
		Return SetError(@error, 0, 0)
	Else
		Return 1
	EndIf

EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _EUM_StripContext
; Description ...: Strips all carriage returns (@CR), line feeds (@LF) and pipes (|) from the context data
; Syntax.........: _EUM_StripCRLF($_sContext)
; Parameters ....: $_sContext 	-	String possible holding the unwanted characters
; Return values .: String without any @CR, @LF and | charachters
; Author ........: The Backbone - The Netherlands
; Modified.......: 12/07/2016
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _EUM_StripContext($_sContext)

	Return StringStripCR(StringReplace(StringReplace($_sContext,"|",""), @LF,""))

EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _EUM_WatcherSet
; Description ...: If desireable change the name for the watchernode (for use in script and output)
; Syntax.........: _EUM_WatcherSet($_sNewWatcherNode)
; Parameters ....: $_sNewWatcherNode -	New Name of watchernode, use FQDN if possible
; Return values .: Success      - 	1
;                  Failure      -	0
; Author ........: The Backbone - The Netherlands
; Modified.......: 26/04/2016
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _EUM_WatcherSet($_sNewWatcherNode)

	if $_sNewWatcherNode = "" Then
		Return SetError(@error, 0, 0)
	Else
		$__g_sWatcherNode = $_sNewWatcherNode
		Return 1
	EndIf
EndFunc

#EndRegion Core functions
